package com.cmeeting.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.cmeeting.ad.entity.RobotSecurityUser;
import com.cmeeting.ad.entity.SysDept;
import com.cmeeting.ad.entity.SysUserSync;
import com.cmeeting.ad.util.SecurityUtil;
import com.cmeeting.mapper.primary.RecordTemplatePermissionMapper;
import com.cmeeting.mapper.secondary.SysUserSysMapper;
import com.cmeeting.pojo.MeetingRecordTemplate;
import com.cmeeting.pojo.RecordTemplatePermission;
import com.cmeeting.service.MeetingRecordTemplateService;
import com.cmeeting.service.RecordTemplatePermissionService;
import com.cmeeting.vo.RecordTemplateVO;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
public class RecordTemplatePermissionServiceImpl extends ServiceImpl<RecordTemplatePermissionMapper, RecordTemplatePermission>  implements RecordTemplatePermissionService {

    @Resource
    private MeetingRecordTemplateService recordTemplateService;
    @Resource
    private SysUserSysMapper sysUserSysMapper;

    /**
     * 授权
     * @param vo
     */
    @Override
    public boolean auth(RecordTemplateVO.TemplatePermissionVO vo) {
        List<RecordTemplateVO.PermissionItem> permissionItems = vo.getPermissionItems();
        if(vo.getTemplateId() == null){
            return false;
        }
        MeetingRecordTemplate template = recordTemplateService.getById(vo.getTemplateId());
        if(template == null){
            throw new RuntimeException("模板不存在");
        }
        List<RecordTemplatePermission> permissions = new ArrayList<>();
        LocalDateTime now = LocalDateTime.now();
        for (RecordTemplateVO.PermissionItem permissionItem : permissionItems) {
            RecordTemplatePermission permission = RecordTemplatePermission.builder()
                    .templateId(vo.getTemplateId())
                    .templateType(template.getType())
                    .relType(permissionItem.getRelType())
                    .relId(permissionItem.getRelId())
                    .createTime(now)
                    .build();
            permissions.add(permission);
        }
        //对比数据库中已保存的权限信息，删除本次授权不包含的对象权限
        List<String> permissionRelIds = permissions.stream().map(RecordTemplatePermission::getRelId).collect(Collectors.toList());
        List<RecordTemplatePermission> savedPermissions = baseMapper.selectList(new LambdaQueryWrapper<RecordTemplatePermission>().eq(RecordTemplatePermission::getTemplateId, template.getId()));
        //过滤出需要删除的权限
        List<Integer> deletablePermissionIds = savedPermissions.stream()
                .filter(item -> !permissionRelIds.contains(item.getRelId())).map(RecordTemplatePermission::getId).collect(Collectors.toList());
        if(!CollectionUtils.isEmpty(deletablePermissionIds)){
            baseMapper.deleteBatchIds(deletablePermissionIds);
        }
        if(!CollectionUtils.isEmpty(permissions)){
            baseMapper.batchInsert(permissions);
        }
        return true;
    }

    /**
     * 查询某个模板已经授权的人员或部门
     * @param vo
     * @return
     */
    @Override
    public List<RecordTemplateVO.PermissionItem> authList(RecordTemplateVO.TemplateAuthVO vo) {
        RobotSecurityUser user = SecurityUtil.getUser();
        List<RecordTemplatePermission> permissions = baseMapper.selectList(
                new LambdaQueryWrapper<RecordTemplatePermission>()
                        .eq(RecordTemplatePermission::getTemplateId,vo.getTemplateId()));
        List<RecordTemplateVO.PermissionItem> permissionItems = new ArrayList<>();
        if(!CollectionUtils.isEmpty(permissions)){
            List<SysUserSync> simpleUserList = sysUserSysMapper.getSimpleUserList(user.getTenantId());
            Map<String, String> simpleUserMap = CollectionUtils.isEmpty(simpleUserList)
                    ? new HashMap<>() : simpleUserList.stream().collect(Collectors.toMap(SysUserSync::getUserId, SysUserSync::getName));
            List<SysDept> simpleDeptList = sysUserSysMapper.getSimpleDeptList(user.getTenantId());
            Map<String, String> simpleDeptMap = CollectionUtils.isEmpty(simpleDeptList)
                    ? new HashMap<>() : simpleDeptList.stream().collect(Collectors.toMap(SysDept::getId, SysDept::getName));
            for (RecordTemplatePermission permission : permissions) {
                String relName = "1".equals(permission.getRelType()) && simpleDeptMap.containsKey(permission.getRelId())
                        ? simpleDeptMap.get(permission.getRelId())
                        : "2".equals(permission.getRelType()) && simpleUserMap.containsKey(permission.getRelId())
                        ? simpleUserMap.get(permission.getRelId()) : permission.getRelId();
                RecordTemplateVO.PermissionItem permissionItem = RecordTemplateVO.PermissionItem.builder()
                        .relId(permission.getRelId())
                        .relType(permission.getRelType())
                        .relName(relName).build();
                permissionItems.add(permissionItem);
            }

        }
        return permissionItems;
    }
}